**本章目录:**

- 0x01 Go语言基础之包

- - 1.包的定义
  - 2.包的导入
  - 3.包的可见性
  - 4.自定义包名
  - 5.匿名导入包
  - 6.包init()初始化函数
  - 7.示例演示

![img](assets/4aa545dccf7de8d4a93c2b2b8e3265ac0a26d216.png@progressive.webp)

# 0x01 Go语言基础之包

描述: 在工程化的Go语言开发项目中，Go语言的源码复用是建立在包（package）基础之上的, 可以提高开发效率，使用其他开发者已经写好的代码（站在巨人的肩膀上）。

# 1.包的定义

描述: Go语言的包`（package）`是多个Go源码的集合，是一种高级的代码复用方案，Go语言为我们提供了很多内置包，如`fmt、os、io`等。

我们还可以根据自己的需要创建自己的包，一个包可以简单理解为一个存放`.go`文件的文件夹。

该文件夹下面的所有go文件都要在代码的第一行添加如下代码声明该文件归属的包。

```go
package 包名

// graphical.go
package area
```

**注意事项:**

- 一个文件夹下面直接包含的文件只能归属一个package，同样一个package的文件不能在多个文件夹下。
- 包名可以不和文件夹的名字一样,`但可以与.go文件名称一致`，包名不能包含 - 符号并且严格按照变量命名的规则进行。
- 在导入包时应该从包的GOPATH/src后的路径开始写起其以`/`作为分隔符。
- 包名为main的包为应用程序的入口包，这种包编译后会得到一个可执行文件，而编译不包含main包的源代码则不会得到可执行文件。



# 2.包的导入

描述: 要在代码中引用其他包的内容，需要使用import关键字导入使用的包。具体语法如下:

import "包的路径"

**示例演示**

```go
// 单行导入的格式如下:
import "包1"
import "包2"

// 多行导入的格式如下:
import (
  "包1"
  "包2"
)

// 实际案例
import (
  "fmt"       // golang 内置包
  "math/rand" // golang 内置包
  "github.com/mattn/go-sqlite3" // golang 项目的工程组织规范
)
```



Tips: 使用`go get`导入github上的package, 以 go-sqlite3 为例，采用go get将package进行下载`go get github.com/mattn/go-sqlite3`，此时该包对应的物理路径是 `$GOPATH/src/github.com/mattn/go-sqlite3`, 此外在你也可以手动进行下载项目到`$GOPATH/src`

**注意事项：**

- import导入语句通常放在文件开头包声明语句的下面。
- 导入的包名需要使用双引号包裹起来,并且如果是多个包需要使用`()`进行包含。
- 包名是从`$GOPATH/src/`后开始计算的，使用/进行路径分隔。
- Go语言中禁止循环导入包。



# 3.包的可见性

描述: 如果想在一个包中引用另外一个包里的标识符`（如变量、常量、类型、函数等）`时，该标识符必须是对外可见的（public）。

在Go语言中只需要将标识符的首字母大写就可以让标识符对外可见了。

举个例子， 我们定义一个包名为pkg2的包，代码如下：



```go
package pkg2
import "fmt"
// # 包变量可见性探究

// 1.首字母小写，外部包不可见，只能在当前包内使用
var a = 100
// 2.首字母大写外部包可见，可在其他包中使用
const Mode = 1

// 3.首字母大写，外部包可见，可在其他包中使用
func Add(x, y int) int {
	return x + y
}
// 4.首字母小写，外部包不可见，只能在当前包内使用
func age() {
	var Age = 18 // 函数局部变量，外部包不可见，只能在当前函数内使用
	fmt.Println(Age)
}
// 5.首字母小写，外部包不可见，只能在当前包内使用
type person struct {
	name string
}
// 6.结构体中的字段名和接口中的方法名如果首字母都是大写，外部包可以访问这些字段和方法
type Student struct {
	Name  string //可在包外访问的方法
	class string //仅限包内访问的字段
}
type Payer interface {
	init() //仅限包内访问的方法
	Pay()  //可在包外访问的方法
}
```



# 4.自定义包名

描述: 在导入包名的时候我们还可以为导入的包设置别名。通常用于导入的包名太长或者导入的包名冲突的情况。

具体语法格式如下：

import 别名 "包的路径"

**示例演示:**

```go
// 1.单行导入方式定义别名：
import "fmt"
import calc "github.com/studygo/pkg_test"
func main() {
	fmt.Println(calc.Add(100, 200))
	fmt.Println(calc.Mode)
}

// 2.多行导入方式定义别名：
import (
    "fmt"
    m "github.com/studygo/pkg_test"
 )
func main() {
	fmt.Println(m.Add(100, 200))
	fmt.Println(m.Mode)
}
```



# 5.匿名导入包

描述: 如果只希望导入包，而不使用包内部的数据时，可以使用匿名导入包。

具体的格式如下：`import _ "包的路径"`

Tips: 匿名导入的包与其他方式导入的包一样都会被编译到可执行文件中。

**补充说明:**
我们可以通过如下格式省略包前前缀,使用想过与同一个.go文件函数类似，但是常常不建议这样使用，可以会与当前文件中的某些相同方法的冲突。

具体的格式如下：`import . "包的路径"`, 示例如下

```go
import (
  . "fmt"
)

func main() {
  Println("我是fmt内置包的函数....")
}
```



# 6.包init()初始化函数

描述: 在Go语言程序执行时导入包语句会自动触发包内部`init()`函数的调用。

**语法格式:**

```go
package custompackage
func init() {
  fmt.Println("custompackage init() execute....")
}
```

**init()函数执行顺序**

- 通常包初始化执行的顺序，如下图所示：

![WeiyiGeek.包init函数执行时机](assets/e0340a28184fc7c34680178c16c61fc984a7ce99.png@942w_590h_progressive.webp)

- 但是实际项目中，Go语言包会从main包开始检查其导入的所有包，每个包中又可能导入了其他的包。Go编译器由此构建出一个树状的包引用关系，再根据引用顺序决定编译顺序，依次编译这些包的代码。
  在运行时，被最后导入的包会最先初始化并调用其init()函数，如下图示：

![WeiyiGeek.多包中初始化函数执行顺序](assets/101eef1b4984a49abfc27a360911e81eed92d771.png@942w_512h_progressive.webp)



**注意事项:**

- init() 函数没有参数也没有返回值。
- init() 函数在程序运行时自动被调用执行，不能在代码中主动调用它。

# 7.示例演示

**工程项目结构:**

```go
// 自定义包的.go文件
➜  pkg pwd & ls
/home/weiyigeek/app/program/project/go/src/weiyigeek.top/custom/pkg/demo1  //(from $GOPATH))
demo1.go

// 调用自定义包的.go文件
➜  pkg ls
weiyigeek.top/studygo/Day04/packagemain.go
```



不多说上代码：

```go
// # weiyigeek.top/custom/pkg/demo1/demo1.go #
package demo

import "fmt"

// 静态变量
const FLAG = true

// 基础变量
var Flag = 1

// 包初始化函数
func init() {
	fmt.Println("This is a package demo ")
	Flag = 1024 // 注意点
}

// 包函数
func Show() {
	var msg = " 我是函数内部的变量 "
	fmt.Printf("FLAG => %v, Flag => %v\nmsg:%v\n", FLAG, Flag, msg)
}

// 结构体
type Person struct{ Name string }

func (p Person) paly() {
	fmt.Printf("%v 正在打游戏....", p.Name)
}

// 接口
type IPerson interface{ paly() }

func Exec(i IPerson) {
	i.paly()
}
```



调用自定义包.go文件:

```go
package main
import (
	"fmt" 	// 不建议如. "fmt" 此使用
	demo "weiyigeek.top/custom/pkg/demo1"
)

func main() {
	fmt.Println(demo.Flag)
	fmt.Println(demo.FLAG)
	demo.Show()
	demo.Exec(demo.Person{Name: "Weiyieek"})
}
```

执行结果:

```go
This is a package demo
1024
true
FLAG => true, Flag => 1024
msg: 我是函数内部的变量
Weiyieek 正在打游戏....
```



**包总结:**

- 我们可以在`GOPATH/src`路径下按照golang项目工程组织规范进行创建自定义包。
- 自定义包中需要外部调用访问的（如变量、常量、类型、函数等），必须首字母进行大写。
- 导入自定义包时我们可以自定义别名,但是需要满足命名规则已经不能与当前目录名称重名。
- 多个包都定义init()函数时，从调用的最后一包中递归向上执行输出。



**错误说明:**

- Step 1.引入自定义包的时候报 go.mod file not found in current directory or any parent directory 错误.

```go
go: go.mod file not found in current directory or any parent directory; see 'go help modules' (exit status 1)
no required module provides package weiyigeek.top/custom/pkg/demo1; to add it: go get weiyigeek.top/custom/pkg/demo1 (compile)go-staticcheck
```

问题原因: go的环境设置问题，其次查看`GOPATH`目录中src为存放第三方go包。
解决办法: `go env -w GO111MODULE=auto`

- Step 2.main redeclared in this block (see details)compiler
  错误信息:

```go
main redeclared in this block (see details)compilerDuplicateDecl 03datatype.go(151, 6): main redeclared in this block
```

原因分析: 在学习study go时候会创建许多.go文件,并且在同一个目录下每个.go的文件里面都有package main,也就是main函数,这就是问题所在。
解决办法: 同一个目录下面不能有多个package main，调整或者创建多个文件夹分别放入对应的文件下执行即可。